package ethz.ch.pp.streams;

import java.util.*;
import java.util.stream.*;
import java.util.concurrent.*;

// This example provides some performance measurements.
// To get good results, switch off JIT Compilation with the following VM Flag:
// -Djava.compiler=NONE
public class FunctionalExample {
    private static float throwaway;

    private static int sleep(int minMillis, int... optMaxMillis){
        int sleepTime = minMillis + (optMaxMillis.length > 0
                ? ThreadLocalRandom.current().nextInt(optMaxMillis[0] - minMillis) : 0);
        try { Thread.sleep(sleepTime); } catch (InterruptedException e){
            System.out.println("The Thread got interrupted: " + Thread.currentThread());
        }
        System.out.println(Thread.currentThread().getName() + " - [" + sleepTime + "ms]");
        return sleepTime;
    }

    public static void main(String[] args) throws Exception {

        System.out.println("Measuring t1... ");
        double t1 = IntStream.range(0,10)
                .map(x -> sleep(200))
                .sum();

        // Set # processors here
        int p = 4;

        System.out.println("\nMeasuring tp...");

        double start = System.currentTimeMillis();
        double total_time = new ForkJoinPool(p).submit(() ->
                IntStream.range(0,10)
                        .parallel()
                        .map(x -> sleep(200))
                        .sum()).get();
        double tp = System.currentTimeMillis() - start;

        System.out.println("\nT1: " + t1 + "ms");
        System.out.println("Tp: " + tp + "ms (for p = " + p + ")");
        System.out.println("Speedup: " + t1 / tp);
        System.out.println("Efficiency: " + (t1 / tp) / p);

    }

}
