package ethz.ch.pp.streams;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.IntStream;

public class Playground {

    // This example, when executed, shows how the collector has to combine results
    // From subtasks (divide and conquer style). This by logging invocations to
    // the two functions that are used to combine results, the first on that
    // appends a single element to an existing list (used mainly in the sequential
    // processing) and the combination of two lists, which is invoked when performing
    // parallel computation.
    public static void main(String[] args) throws InterruptedException {
        List<Integer> squares = IntStream.range(0, 10)
                .parallel()
                .map(x -> x * x)
                .peek(System.out::println)
                .collect(
                        ArrayList::new,
                        (integers, i) -> {
                            integers.add(i);
                            System.out.println("Adding " + i + " to " + integers);
                        },
                        (integers, integers2) -> {
                            System.out.println("Adding " + integers2 + " to " + integers);
                            integers.addAll(integers2);
                        }
                        );
        System.out.println(squares);
    }
}
