package ethz.ch.pp.sum;

import java.util.Random;

public class TestRunner {

    private final int times = 100;
    private final int[] testData;
    private int expected;
    private long t1;

    public static void main(String[] args) {
        new TestRunner().test();
    }

    public TestRunner() {
        testData = setup(new int[1_000_000]);
    }

    public void test() {
        int processors = 4;
        measureT1();
        measureTp(new SumWithThreads(), processors);
        measureTp(new SumWithExecutor(), processors);
        measureTp(new SumWithForkJoin(), processors);
        measureTp(new SumWithStreams(), processors);
    }
  
    private int[] setup(int[] data){
        System.out.print("Setting up testdata with " + data.length + " elements. ");
        System.out.println("Testing " + times + " times.");
        Random rd = new Random();
        for(int i = 0; i < data.length; ++i){
            int nextNumber = rd.nextInt(10);
            data[i] = nextNumber;
        }
        return data;
    }

    private int sequentialSum(int[] data){
        int ans = 0;
        for (int elem : data) {
            ans += elem;
        }
        return ans;
    }

    private void measureT1() {
        System.out.print("Measuring T1: ");
        long start = System.currentTimeMillis();
        expected=0;
        for (int i = 0; i < times; ++i){
            expected += sequentialSum(testData);
        }
        t1 = System.currentTimeMillis() - start;
        System.out.println(t1 + "ms");
    }

    private void measureTp(Sum technology, int p) {
        System.out.println("\n---- Using " + technology.getClass().getName() + " with p = " + p + " ----");
        System.out.print("Measuring Tp: ");
        long start = System.currentTimeMillis();
        int actual = 0;
        try {
            for (int i = 0; i < times; ++i) {
                actual += technology.sum(testData, p);
            }
        } catch (Exception e){
            e.printStackTrace();
        }
        long tp = System.currentTimeMillis() - start;
        System.out.println(tp + "ms");

        double speedup = (double)t1/tp;
        System.out.printf("Speedup     : %.2f\n", speedup);
        double efficiency = speedup / p;
        System.out.printf("Efficiency  : %.2f%%\n", efficiency*100);

        assert expected == actual: "Parallel version didn't get the correct result. Expected " + expected + ", actual: " + actual;
    }

}
